from __future__ import unicode_literals

#from ngofile.cli import main

#def test_main():
#    runner = CliRunner()
#    result = runner.invoke(main, [])
#
#    assert result.output == '()\n'
#    assert result.exit_code == 0
